/*******************************************************************************
 * Copyright 2012 Mohamed Sellami, Telecom SudParis
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package telecom.sudparis.eu.paas.core.server.ressources.manager.application;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.zip.ZipFile;
import javax.ws.rs.Path;
import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;
import javax.ws.rs.core.Response.Status;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.IOUtils;

import com.sun.jersey.core.header.ContentDisposition;
import com.sun.jersey.multipart.FormDataBodyPart;

import telecom.sudparis.eu.paas.api.ressources.manager.application.RestApplicationManager;
import telecom.sudparis.eu.paas.core.server.pool.application.ApplicationPool;
import telecom.sudparis.eu.paas.core.server.ressources.exception.CloudSpaceException;
import telecom.sudparis.eu.paas.core.server.ressources.exception.NotSupportedException;
import telecom.sudparis.eu.paas.core.server.xml.Error;
import telecom.sudparis.eu.paas.core.server.xml.OperationResponse;
import telecom.sudparis.eu.paas.core.server.xml.application.ApplicationType;


/**
 * REST resource of type ApplicationManager
 * 
 * @author Mehdi Ahmed-Nacer (Telecom SudParis)
 * 
 */

@Path("app")
public class ApplicationManagerRessource implements RestApplicationManager {

	private OperationResponse or = new OperationResponse();
	private CloudFoundaryManageApplication cf;
	@Override
	public Response createApplication(String cloudApplicationDescriptor) {
		ApplicationType appCf = null, appAws=null;

		try{
			cf = new CloudFoundaryManageApplication();
			appCf = cf.createApplication(cloudApplicationDescriptor);
		}catch(Exception e){
			throw new NotSupportedException("Exception in CloudFoundaryManageApplication. "+e.getMessage());
		}

		try{
			AmazonManageApplication aws = new AmazonManageApplication();
			appAws = aws.createApplication(cloudApplicationDescriptor);
		}catch(Exception e){
			throw new NotSupportedException("Exception in AmazonManageApplication. "+e.getMessage());
		}


		or.setValue("AppId CF: "+ appCf.getAppId()+", AWS: "+appAws.getAppId());
		//or.setValue("AppId AWS: "+ appCf.getAppId());
		return Response.status(Response.Status.OK).entity(or).build();

		/*Set<GenericEntity<ApplicationType>> set = new HashSet<GenericEntity<ApplicationType>>();
		set.add(new GenericEntity<ApplicationType>(appCf){});
		set.add(new GenericEntity<ApplicationType>(appAws){});
		return Response.status(Response.Status.OK)
                .entity(set)
                .type(MediaType.APPLICATION_XML_TYPE)
                .build();*/

		/*return Response.status(Response.Status.OK)
				.entity(new GenericEntity<ApplicationType>(appCf) {
				}).type(MediaType.APPLICATION_XML_TYPE).build();*/
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response findApplications() {
		throw new NotSupportedException(
				"The findApplications is not yet implemented.");
	}

	/**
	 * {@inheritDoc}
	 */

	public Response startApplication(String appid) {
		throw new NotSupportedException(
				"The startApplication is not yet implemented.");

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response describeApplication(String appid) {
		throw new NotSupportedException(
				"The describeApplication is not yet implemented.");

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response deleteApplication(String appid) {
		throw new NotSupportedException(
				"The deleteApplication is not yet implemented.");

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response deleteApplications() {
		throw new NotSupportedException(
				"The deleteApplications is not yet implemented.");

	}

	/**
	 * {@inheritDoc}
	 */

	public Response stopApplication(String appid) {
		throw new NotSupportedException(
				"The stopApplication is not yet implemented.");

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response updateApplication(String appid,
			String cloudApplicationDescriptor) {
		throw new NotSupportedException(
				"The update Application is not yet implemented.");
	}

	/**
	 * {@inheritDoc}
	 */

	public Response restartApplication(String appid) {
		throw new NotSupportedException(
				"The restartApplication is not yet implemented.");

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response deployApplication(String appid, String envid,
			InputStream uploadedInputStream) {


		return Response.status(Response.Status.OK)
				.entity("Not in correct deploy").type(MediaType.APPLICATION_XML_TYPE).build();
	}


	@Override
	public Response MultideployApplication(String appid, String envid, List<FormDataBodyPart> uploadedInputStream) {
		
		/*or.setValue("Before");
		if(true){
			return Response.status(Response.Status.OK)
					.entity(or).type(MediaType.APPLICATION_XML_TYPE).build();
		}*/

		String[] idApp = appid.split("-");
		String[] idEnv = envid.split("-");
		InputStream app1 = uploadedInputStream.get(0).getEntityAs(InputStream.class);
		InputStream app2 = uploadedInputStream.get(1).getEntityAs(InputStream.class);

		CloudFoundaryManageApplication cf = new CloudFoundaryManageApplication(idApp[0], idEnv[0], app1);
		cf.start();//deployment upon CF by a Thread

		AmazonManageApplication aws = new AmazonManageApplication(idApp[1], idEnv[1], app2);
		aws.start();//deployment upon AWS by a Thread
		
		try {
			cf.join();
			aws.join();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		or.setValue("Successfully deployment");

		return Response.status(Response.Status.OK)
				.entity(or).type(MediaType.APPLICATION_XML_TYPE).build();

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Response undeployApplication(String envid, String appid) {
		throw new NotSupportedException();
	}


	private static String prettyFormat(String input, int indent) {
		try {
			Source xmlInput = new StreamSource(new StringReader(input));
			StringWriter stringWriter = new StringWriter();
			StreamResult xmlOutput = new StreamResult(stringWriter);
			TransformerFactory transformerFactory = TransformerFactory
					.newInstance();
			transformerFactory.setAttribute("indent-number", indent);
			Transformer transformer = transformerFactory.newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty(OutputKeys.STANDALONE, "yes");
			transformer.transform(xmlInput, xmlOutput);
			return xmlOutput.getWriter().toString();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}



	private static void keepTrace(String text){
		try {
			PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter("myfile.txt", true)));
			out.println(text);
			out.close();
		} catch (IOException e) {
			//exception handling left as an exercise for the reader
		}
	}
}

