package telecom.sudparis.eu.paas.core.server.ressources.manager.application;


import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import java.util.zip.ZipFile;

import javax.ws.rs.Path;
import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

//import org.apache.commons.httpclient.methods.GetMethod;
import org.cloudfoundry.client.lib.CloudCredentials;
import org.cloudfoundry.client.lib.CloudFoundryClient;
import org.cloudfoundry.client.lib.archive.ApplicationArchive;
import org.cloudfoundry.client.lib.archive.DirectoryApplicationArchive;
import org.cloudfoundry.client.lib.archive.ZipApplicationArchive;
import org.cloudfoundry.client.lib.domain.CloudApplication;
import org.cloudfoundry.client.lib.domain.CloudEntity;
import org.cloudfoundry.client.lib.domain.CloudService;
import org.cloudfoundry.client.lib.domain.CloudSpace;
import org.cloudfoundry.client.lib.domain.Staging;
import org.springframework.web.client.HttpServerErrorException;

import telecom.sudparis.eu.paas.core.server.artifact.handling.AppObj;
import telecom.sudparis.eu.paas.core.server.artifact.handling.AppProcessing;
import telecom.sudparis.eu.paas.core.server.pool.application.ApplicationPool;
import telecom.sudparis.eu.paas.core.server.pool.environment.EnvironmentPool;
import telecom.sudparis.eu.paas.core.server.ressources.exception.CloudSpaceException;
import telecom.sudparis.eu.paas.core.server.ressources.util.ApplicationLinkGenerator;
import telecom.sudparis.eu.paas.core.server.ressources.util.FileUtilities;
import telecom.sudparis.eu.paas.core.server.xml.Error;
import telecom.sudparis.eu.paas.core.server.xml.LinksListType;
import telecom.sudparis.eu.paas.core.server.xml.OperationResponse;
import telecom.sudparis.eu.paas.core.server.xml.application.ApplicationType;
import telecom.sudparis.eu.paas.core.server.xml.application.DeployableType;
import telecom.sudparis.eu.paas.core.server.xml.application.InstanceType;
import telecom.sudparis.eu.paas.core.server.xml.application.InstancesType;
import telecom.sudparis.eu.paas.core.server.xml.application.UrisType;
import telecom.sudparis.eu.paas.core.server.xml.environment.ConfigurationType;
import telecom.sudparis.eu.paas.core.server.xml.environment.EntryType;
import telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationComponentType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationDeployableType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationManifest;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationVersionInstanceType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationVersionType;

/**
 * 
 * @author Mehdi Ahmed-Nacer (Telecom SudParis)
 * 
 */

public class CloudFoundaryManageApplication extends Thread {
	private static ResourceBundle rb = ResourceBundle
			.getBundle("telecom.sudparis.eu.paas.core.server.ressources.credentials_cf");
	private static final String ccUrl = rb.getString("vcap.target");
	private static String apiUrl = rb.getString("api.public.url");
	private static final String TEST_USER_EMAIL = rb.getString("vcap.email");
	private static final String TEST_USER_PASS = rb.getString("vcap.passwd");
	private static final String CLOUD_SPACE = rb.getString("vcap.space");
	private static final String HOST_NAME = rb.getString("host");
	private static final boolean CHECK_EXISTS = true;
	private OperationResponse or = new OperationResponse();
	private Error error = new Error();
	private static boolean checkCFApplications = false;
	private static String localTempPath = System.getProperty("java.io.tmpdir");
	public static String val = "";
	
	String appwar, envwar;
	InputStream war;
	public CloudFoundaryManageApplication(){

	}
	public CloudFoundaryManageApplication(String app, String env, InputStream war){
		this.appwar = app;
		this.envwar = env;
		this.war = war;
	}

	
	/***************************************************************************************/
	/**************************** createApplication methods ***************************/
	/***************************************************************************************/
	
	public ApplicationType createApplication(String cloudApplicationDescriptor) {
		ApplicationType app = new ApplicationType();
		try {
			copyDeployedApps2Pool();
			if (cloudApplicationDescriptor != null
					&& !cloudApplicationDescriptor.equals("")) {
				InputStream is = new ByteArrayInputStream(
						cloudApplicationDescriptor.getBytes());
				JAXBContext jaxbContext;
				PaasApplicationManifest manifest = new PaasApplicationManifest();
				try {
					jaxbContext = JAXBContext
							.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");
					Unmarshaller jaxbUnmarshaller = jaxbContext
							.createUnmarshaller();
					JAXBElement<PaasApplicationManifest> root = jaxbUnmarshaller
							.unmarshal(new StreamSource(is),
									PaasApplicationManifest.class);
					manifest = root.getValue();

				} catch (JAXBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				// create the application object
				
				// retrieve appName
				PaasApplicationComponentType component = 
						manifest.getPaasApplication().getPaasApplicationComponent().get(0);
				// retrieve appName
				String appName = component.getName();
				app.setAppName(appName);

				// retrieve app description
				String description = component.getDescription().toString();
				app.setDescription(description);

				// retrieve uris
				// List<String> uris = new ArrayList<String>();
				// uris.add(appName + "." + HOST_NAME);
				UrisType uris = new UrisType();
				uris.getUri().add(appName + "." + HOST_NAME);
				app.setUris(uris);

				// generate appID
				Long id = getNextId();
				app.setAppId((int) (long) id);

				// retrieve deployableName
				PaasApplicationDeployableType deployable = 
						component.getPaasApplicationVersion().get(0).getPaasApplicationDeployable().get(0);
				String deployableName = deployable.getName();

				// retrieve deployableType
				String deployableType = deployable.getContentType();

				// retrieve deployableDirectory or deployableId
				String deployableId = null;
				String deployableDirectory = deployable.getLocation();
				// if the deployable is already in the DB we provide it's ID
				DeployableType dep = new DeployableType();
				if (!(deployableDirectory.contains("/") || deployableDirectory
						.contains("\\"))) {
					dep.setDeployableId(deployableDirectory);
				}
				// Define the deployable element
				else {

					dep.setDeployableDirectory(deployableDirectory);

				}
				dep.setDeployableName(deployableName);
				dep.setDeployableType(deployableType);
				app.setDeployable(dep);

				// retrieve ApplicationVersionInstance Names and number
				PaasApplicationVersionType version = component.getPaasApplicationVersion().get(0);
				List<PaasApplicationVersionInstanceType> listOfInstances = version.getPaasApplicationVersionInstance();
				InstancesType listVI = new InstancesType();
				int nbInstances = 0;

				if (listOfInstances != null && listOfInstances.size() > 0)
					for (PaasApplicationVersionInstanceType p : listOfInstances) {
						if (p != null) {
							InstanceType vi = new InstanceType();
							vi.setInstanceName(p.getName());
							listVI.getInstance().add(vi);
							nbInstances++;
						}
					}

				// Update the Application object with the
				// Adequate LinkList
				LinksListType linksList = new LinksListType();
				linksList = ApplicationLinkGenerator.addDescribeAppLink(
						linksList, id.toString(), apiUrl);
				linksList = ApplicationLinkGenerator.addCreateAppLink(
						linksList, apiUrl);
				linksList = ApplicationLinkGenerator.addDestroyAppLink(
						linksList, id.toString(), apiUrl);
				linksList = ApplicationLinkGenerator.addFindAppsLink(linksList,
						apiUrl);
				linksList = ApplicationLinkGenerator.addUpdateAppLink(
						linksList, id.toString(), apiUrl);
				app.setLinksList(linksList);
				app.setInstances(listVI);
				app.setNbInstances(nbInstances);
				// app.setCheckExists(CHECK_EXISTS);
				app.setStatus("CREATED");
				
				ApplicationPool.INSTANCE.add(app);
				
				return app;
			} else {
				System.out
				.println("Failed to retrieve the cloud Application Descriptor");
				error.setValue("Failed to retrieve the cloud Application Descriptor.");
				return app;
			}
		} catch (Exception e) {
			System.out.println("Failed to create the application: "
					+ e.getMessage());
			e.printStackTrace();
			error.setValue("Failed to create the application: "
					+ e.getMessage());
			return app;
		}
	}

	
	/***************************************************************************************/
	/**************************** deployApplication methods ***************************/
	/***************************************************************************************/
	public Response deployApplication(String appid, String envid,
			InputStream uploadedInputStream) {
try {
			
			telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType env = EnvironmentPool.INSTANCE
					.getEnv(envid);
			ApplicationType app = ApplicationPool.INSTANCE.getApp(appid);

			CloudFoundryClient client = null;
			List<String> servicesList = getEnvironmentServiceList(env);
			// List<String> servicesList = env.getServiceNames();

			try {
				 CloudCredentials credentials = new CloudCredentials(TEST_USER_EMAIL, TEST_USER_PASS);
				 client = new CloudFoundryClient(credentials, getTargetURL(ccUrl));
				 client.login();
				 CloudSpace theCloudSpace=getSpace(client);
				 if (theCloudSpace==null)
					 throw new  CloudSpaceException(theCloudSpace.getName());
				client = new CloudFoundryClient(new CloudCredentials(
						TEST_USER_EMAIL, TEST_USER_PASS), new URL(ccUrl),theCloudSpace);
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			client.login();
			/*or.setValue("After connection");
			if(true)
				return Response.status(Response.Status.OK).entity(or)
						.type(MediaType.APPLICATION_XML_TYPE).build();*/
			// We start by uploading the application files

			/**
			 * TODO getDeployableId getRemoteAppStreamByID
			 */

			String artefactId = app.getDeployable().getDeployableId();
			String uploadedFileLocation = localTempPath + "/"
					+ app.getDeployable().getDeployableName();
			AppObj appObj = null;
			AppProcessing appProcessing = new AppProcessing(localTempPath
					+ "/");
			if (artefactId == null) {
				FileUtilities.writeToFile(uploadedInputStream,
						uploadedFileLocation);
				InputStream is = new FileInputStream(new File(
						uploadedFileLocation));

				appObj = appProcessing.uploadAppToRemoteServer(is, app
						.getDeployable().getDeployableName());
				artefactId = appObj.getId();
				System.out.println("ID ID:" + artefactId);
			} else {
				FileUtilities.writeToFile(
						appProcessing.getRemoteAppStreamByID(artefactId),
						uploadedFileLocation);
			}

			// redefine the deployable element according to the uploaded
			// file location
			DeployableType newDep = new DeployableType();
			newDep.setDeployableDirectory(localTempPath);
			newDep.setDeployableName(app.getDeployable()
					.getDeployableName());
			newDep.setDeployableType(app.getDeployable()
					.getDeployableType());
			newDep.setDeployableId(artefactId);
			// update the application pool
			ApplicationPool.INSTANCE.updateApp(appid, newDep);

			// String runtime = env.getStaging().map.get("runtime");
			String runtime = getEnvironmentRuntime(env);

			String framework = getEnvironmentFramework(env);

			Staging staging = new Staging(runtime, framework);
			
			List<CloudService> servicesLst = client.getServices();

			// Services Creation if not already exits
			// TODO create the other supported services
			if (servicesList != null && servicesList.size() != 0) {
				for (String serviceName : servicesList) {
					if (serviceName.toLowerCase().contains("sql")) {
						if (!serviceExists(serviceName, client))
							createMySqlService(serviceName, client);
					} else if (serviceName.toLowerCase().contains("redis")) {
						if (!serviceExists(serviceName, client))
							createRedisService(serviceName, client);
					}
				}
			}
			// get the memory attribute from the environment
			// app.setMemory(env.getEnvMemory());
			// create the application
	
			for (String s:servicesList)
				System.out.println(s);
			client.createApplication(app.getAppName(), staging,
					(int) env.getEnvMemory(), app.getUris().getUri(),
					servicesList);
		
			// Sets the number of instances
			if (app.getNbInstances() > 0)
				client.updateApplicationInstances(app.getAppName(),app.getNbInstances());

			// Services binding
			/*if (servicesLst != null && servicesLst.size() != 0)
				for (String serviceName : servicesList) {
					if (serviceName != null) {
						client.bindService(app.getAppName(), serviceName);
					}
				}*/

			// TODO upload Application we treated only the case of an
			// artifact
			if (app.getDeployable().getDeployableType().equals("artifact")) {
				String artefactFileName = app.getDeployable()
						.getDeployableName();
				String artefactFilePath = app.getDeployable()
						.getDeployableDirectory().replace("\\", "/");
				if (!artefactFilePath.endsWith("/"))
					artefactFilePath = artefactFilePath + "/";
				File artefactFile = new File(artefactFilePath
						+ artefactFileName);
				ZipFile zipFile = new ZipFile(artefactFile);
				ApplicationArchive appArchive = new ZipApplicationArchive(
						zipFile);
				try {
					System.out.println("##########"+app.getAppName());
					client.uploadApplication(app.getAppName(), appArchive);
				} catch (HttpServerErrorException e) {
					// this is to avoid timeout exceptions that occurs
					// during
					// the upload of large applications
					// TODO add a method to see the status of the upload
					System.out
							.println("[DeployApplication]: Entering HttpServerErrorException....");
				}
			} else if (app.getDeployable().getDeployableType()
					.equals("folder")) {
				// TODO
				String folderPath = app.getDeployable()
						.getDeployableDirectory().replace("\\", "/");
				File folderFile = new File(folderPath);
				ApplicationArchive archive = new DirectoryApplicationArchive(
						folderFile);
				// client.uploadApplication(app.getAppName(), archive);
				try {
					client.uploadApplication(app.getAppName(), app
							.getDeployable().getDeployableDirectory()
							.replace("\\", "/"));
				} catch (HttpServerErrorException e) {
					// this is to avoid timeout exceptions that occurs
					// during the
					// upload of large applications
					// TODO add a method to see the status of the upload
					System.out
							.println("[DeployApplication]: Entering HttpServerErrorException....");
				}
			}
			client.logout();
			// delete the temp uploaded deployable
			FileUtilities.deletefile(uploadedFileLocation);
			// update the Application with describeApp and deleteApp links
			LinksListType linksList = ApplicationPool.INSTANCE.getApp(appid)
					.getLinksList();
			linksList = ApplicationLinkGenerator.addStartAppLink(linksList,
					appid, apiUrl);
			linksList = ApplicationLinkGenerator.addStopAppLink(linksList,
					appid, apiUrl);
			linksList = ApplicationLinkGenerator.addRestartAppLink(linksList,
					appid, apiUrl);

			ApplicationPool.INSTANCE.updateApp(appid, linksList);
			// adds the envID to the deployed application
			ApplicationPool.INSTANCE.updateApp(appid, envid);

			return Response.status(Response.Status.OK).entity(app)
					.type(MediaType.APPLICATION_XML_TYPE).build();
		} catch (Exception e) {
			System.out.println("Failed to deploy the application: "
					+ e.getMessage());
			e.printStackTrace();
			error.setValue("Failed to deploy the application: "
					+ e.getMessage());
			return Response.status(Response.Status.INTERNAL_SERVER_ERROR)
					.entity(error).type(MediaType.APPLICATION_XML_TYPE).build();
		}
	}
	
	/***************************************************************************************/
	/**************************** Private methods ******************************************/
	/***************************************************************************************/
	
	private void copyDeployedApps2Pool() {
		
		if (!checkCFApplications) {
			checkCFApplications = true;
			CloudFoundryClient client = null;
			List<CloudApplication> appListCF = null;
			client = new CloudFoundryClient(new CloudCredentials(
					TEST_USER_EMAIL, TEST_USER_PASS), getTargetURL(ccUrl));
			val = TEST_USER_EMAIL+","+TEST_USER_PASS;	
			client.login();
				appListCF = client.getApplications();
				client.logout();
			// List<Application> appListPool = appPool.INSTANCE.getAppList();
			for (CloudApplication ca : appListCF) {
				String id = Long.toString(getNextId());
				ApplicationType app = new ApplicationType();
				app.setAppId(Integer.parseInt(id));
				app.setAppName(ca.getName());
				// Since we can not recreate an environment, we recover only the
				// configuration
				// elements and the memory
				EnvironmentType env = new EnvironmentType();
				ConfigurationType conf = new ConfigurationType();
				// get the service Names
				for (String service : ca.getServices()) {
					EntryType e = new EntryType();
					e.setKey("service");
					e.setValue(service);
					conf.getEntry().add(e);
				}
				env.setConfiguration(conf);
				env.setEnvMemory(ca.getResources().get("memory"));
				app.setEnvironment(env);
				app.setStatus(ca.getState().toString());
				UrisType uris = new UrisType();
				uris.getUri().addAll(ca.getUris());
				app.setUris(uris);
				app.setNbInstances(ca.getInstances());
				LinksListType linksList = new LinksListType();
				linksList = ApplicationLinkGenerator.addDescribeAppLink(
						linksList, id, apiUrl);
				linksList = ApplicationLinkGenerator.addDestroyAppLink(
						linksList, id, apiUrl);
				linksList = ApplicationLinkGenerator.addCreateAppLink(
						linksList, apiUrl);
				linksList = ApplicationLinkGenerator.addFindAppsLink(linksList,
						apiUrl);
				linksList = ApplicationLinkGenerator.addUpdateAppLink(
						linksList, id, apiUrl);
				linksList = ApplicationLinkGenerator.addStartAppLink(linksList,
						id, apiUrl);
				linksList = ApplicationLinkGenerator.addStopAppLink(linksList,
						id, apiUrl);
				linksList = ApplicationLinkGenerator.addRestartAppLink(
						linksList, id, apiUrl);

				app.setLinksList(linksList);
				ApplicationPool.INSTANCE.add(app);
			}
		}
	}

	private static URL getTargetURL(String target) {
		try {
			return new URI(target).toURL();
		} catch (Exception e) {
			System.out.println("The target URL is not valid: " + e.getMessage());
		}
		System.exit(1);
		return null;
	}

	private synchronized Long getNextId() {
		return new Long(ApplicationPool.INSTANCE.getNextID());
	}
	private List<String> getEnvironmentServiceList(
			telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType env) {
		List<telecom.sudparis.eu.paas.core.server.xml.environment.EntryType> list = env
				.getConfiguration().getEntry();
		List<String> serviceList = new ArrayList<String>();
		System.out.println("HERE display list:" + list);
		if (list != null)
			for (telecom.sudparis.eu.paas.core.server.xml.environment.EntryType entry : list) {
				System.out.println("HERE display entry:" + entry);
				if (entry.getKey().toLowerCase().equals("service"))
					serviceList.add(entry.getValue());
				System.out.println("HERE display entry.getkey:"
						+ entry.getKey());
			}
		return serviceList;
	}

	private String getEnvironmentRuntime(
			telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType env) {
		List<telecom.sudparis.eu.paas.core.server.xml.environment.EntryType> list = env
				.getConfiguration().getEntry();

		for (telecom.sudparis.eu.paas.core.server.xml.environment.EntryType entry : list) {
			if (entry.getKey().toLowerCase().equals("runtime"))
				return entry.getValue();
		}
		return null;
	}

	private String getEnvironmentFramework(
			telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType env) {
		List<telecom.sudparis.eu.paas.core.server.xml.environment.EntryType> list = env
				.getConfiguration().getEntry();

		for (telecom.sudparis.eu.paas.core.server.xml.environment.EntryType entry : list) {
			if (entry.getKey().toLowerCase().equals("framework"))
				return entry.getValue();
		}
		return null;
	}
	private boolean serviceExists(String serviceName, CloudFoundryClient client) {
		if (client.getService(serviceName)==null)
			return false;
		else 
			return true;
	}

	private void createMySqlService(String serviceName,
			CloudFoundryClient client) {
		CloudService service = new CloudService(CloudEntity.Meta.defaultMeta(),
				serviceName);

		service.setLabel("cleardb");
		service.setPlan("spark");
		service.setVersion("n/a");

		client.createService(service);
	}

	private void createRedisService(String serviceName,
			CloudFoundryClient client) {
		CloudService service = new CloudService(CloudEntity.Meta.defaultMeta(),
				serviceName);

		service.setVersion("2.2");
		service.setProvider("core");

		client.createService(service);
	}

	private CloudSpace getSpace(CloudFoundryClient client){
		List<CloudSpace> lst=client.getSpaces();
		CloudSpace cs=null;
		for (CloudSpace c:lst){

			if (c.getName().equalsIgnoreCase(CLOUD_SPACE)){
				cs=c;
			}
		}
		return cs;

	}
	
	public void run() {
		deployApplication(this.appwar, this.envwar, this.war);
	}



}
