package telecom.sudparis.eu.paas.core.server.ressources.manager.environment;


import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ResourceBundle;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

import com.amazonaws.AmazonClientException;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.profile.ProfileCredentialsProvider;
import com.amazonaws.services.ec2.AmazonEC2;
import com.amazonaws.services.ec2.AmazonEC2Client;
import com.amazonaws.services.elasticbeanstalk.AWSElasticBeanstalk;
import com.amazonaws.services.elasticbeanstalk.AWSElasticBeanstalkAsync;
import com.amazonaws.services.elasticbeanstalk.AWSElasticBeanstalkAsyncClient;
import com.amazonaws.services.elasticbeanstalk.AWSElasticBeanstalkClient;
import com.amazonaws.services.elasticbeanstalk.model.CreateConfigurationTemplateRequest;
import com.amazonaws.services.elasticbeanstalk.model.CreateConfigurationTemplateResult;
import com.amazonaws.services.elasticbeanstalk.model.CreateEnvironmentRequest;
import com.amazonaws.services.elasticbeanstalk.model.CreateEnvironmentResult;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3Client;

import telecom.sudparis.eu.paas.core.server.pool.environment.EnvironmentPool2;
import telecom.sudparis.eu.paas.core.server.ressources.util.EnvironmentLinkGenerator;
import telecom.sudparis.eu.paas.core.server.xml.Error;
import telecom.sudparis.eu.paas.core.server.xml.LinksListType;
import telecom.sudparis.eu.paas.core.server.xml.OperationResponse;
import telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationManifest;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasEnvironmentType;

/**
 * 
 * @author Mehdi Ahmed-Nacer (Telecom SudParis)
 * 
 */
public class AmazonManageEnvironment {

	/**
	 * ressource bundle to get the connexion credentials
	 */
	private static ResourceBundle rb = ResourceBundle
			.getBundle("telecom.sudparis.eu.paas.core.server.ressources.credentials_aws");

	/**
	 * The public EB-PaaS API URL
	 */
	private static String apiUrl = rb.getString("api.public.url");

	/**
	 * An element to display The response
	 */
	private OperationResponse or = new OperationResponse();

	/**
	 * An element to display Errors
	 */
	private Error error = new Error();
	/**
	 * The Runtimes supported in the current implementation
	 */
	public static final String JAVA6 = "java";
	public static final String JAVA7 = "java7";

	static AmazonEC2      ec2;
	static AmazonS3       s3;
	static AWSElasticBeanstalkAsync ebas;
	static AWSElasticBeanstalk eb;   

	public AmazonManageEnvironment(){

	}

	public EnvironmentType createEnvironment(String environmentTemplateDescriptor) {

		try {
			init();
			if (environmentTemplateDescriptor != null) {
				InputStream is = new ByteArrayInputStream(
						environmentTemplateDescriptor.getBytes());

				JAXBContext jaxbContext;
				PaasApplicationManifest manifest = new PaasApplicationManifest();
				try {
					jaxbContext = JAXBContext
							.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");
					Unmarshaller jaxbUnmarshaller = jaxbContext
							.createUnmarshaller();

					JAXBElement<PaasApplicationManifest> root = jaxbUnmarshaller
							.unmarshal(new StreamSource(is),
									PaasApplicationManifest.class);
					manifest = root.getValue();

				} catch (JAXBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				PaasEnvironmentType envType = manifest.getPaasApplication().
						getPaasEnvironment().get(1);

				// retrieve envName
				String envName = envType.getName();

				//retrieve appName
				String appName=
						manifest.getPaasApplication().getPaasApplicationComponent().get(1).getName();

				//retrieve templateName
				String tempName=envType.getTemplate();


				//retrieve plateform
				String SolutionStackName = envType.getPaasEnvironmentTemplate().getDescription();		



				/**************************************Creer une configuration Template *************************************/


				CreateConfigurationTemplateRequest request= new CreateConfigurationTemplateRequest(appName,tempName);
				request.setSolutionStackName(SolutionStackName);
				CreateConfigurationTemplateResult  res=eb.createConfigurationTemplate(request);			     	

				/****************Creer un environnement**************************/

				CreateEnvironmentRequest envir=new CreateEnvironmentRequest(appName, envName);

				envir.setSolutionStackName(SolutionStackName);
				envir.setApplicationName(appName);
				envir.setEnvironmentName(envName);
				envir.setCNAMEPrefix(appName+"COAPS"+envName);
				//envir.setTemplateName(res.getTemplateName());
				Future future = ebas.createEnvironmentAsync(envir);

				while (!future.isDone()) {
					Thread.sleep(240000);
					// Do other processing while you're waiting for the response
				}

				try {

					CreateEnvironmentResult r1 = (CreateEnvironmentResult) future.get();
				} catch (ExecutionException ee) {
					// Futures always wrap errors as an ExecutionException.
					// The *real* exception is stored as the cause of the ExecutionException
					Throwable exception = ee.getCause();
					System.out.println("Error creation: " + exception.getMessage());

				}
				// generate envID
				Long id = getNextId();

				EnvironmentType env = new EnvironmentType();
				env.setEnvId((int) (long) id);
				env.setEnvName(envName);
				env.setEnvDesc(SolutionStackName);


				// add the environment STATE and hplinks links
				LinksListType linksList = new LinksListType();
				linksList = EnvironmentLinkGenerator.addGetEnvLink(linksList,
						Long.toString(id), apiUrl);
				linksList = EnvironmentLinkGenerator.addGetEnvsLink(linksList,
						apiUrl);
				linksList = EnvironmentLinkGenerator.addGetInfoLink(linksList,
						apiUrl);
				linksList = EnvironmentLinkGenerator.addnewEnvLink(linksList,
						apiUrl);

				linksList = EnvironmentLinkGenerator.addDestroyEnvLink(
						linksList, Long.toString(id), apiUrl);
				linksList = EnvironmentLinkGenerator.addUpdateEnvLink(
						linksList, Long.toString(id), apiUrl);

				env.setLinksList(linksList);

				EnvironmentPool2.INSTANCE2.add(env);

				return env;
			}
			else {
				System.out
				.println("Failed to retrieve the cloud Environment Descriptor");
				throw new AmazonClientException("Failed to retrieve the cloud Environment Descriptor.");
			}
		} catch (Exception e) {
			System.out.println("Failed to create the environment: "
					+ e.getMessage());
			e.printStackTrace();
			throw new AmazonClientException("Failed to create the environment: "
					+ e.getMessage());
		}
	}

	private static void init() throws Exception {

		AWSCredentials credentials = null;
		try {
			credentials = new ProfileCredentialsProvider("default").getCredentials();
		} catch (Exception e) {
			throw new AmazonClientException(
					"Cannot load the credentials from the credential profiles file. " +
							"Please make sure that your credentials file is at the correct " +
							"location (C:\\Users\\acer\\Desktop\\coaps2), and is in valid format.",
							e);
		}
		ec2 = new AmazonEC2Client(credentials);
		s3  = new AmazonS3Client(credentials);
		eb =  new AWSElasticBeanstalkClient(credentials);
		ebas = new AWSElasticBeanstalkAsyncClient(credentials);

	}

	private synchronized Long getNextId() {
		return new Long(EnvironmentPool2.INSTANCE2.getNextID());
	}

}
