package telecom.sudparis.eu.paas.core.server.ressources.manager.environment;


import java.io.ByteArrayInputStream;


import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

import org.cloudfoundry.client.lib.domain.Staging;

import telecom.sudparis.eu.paas.core.server.pool.environment.EnvironmentPool;
import telecom.sudparis.eu.paas.core.server.ressources.util.EnvironmentLinkGenerator;
import telecom.sudparis.eu.paas.core.server.xml.Error;
import telecom.sudparis.eu.paas.core.server.xml.LinksListType;
import telecom.sudparis.eu.paas.core.server.xml.OperationResponse;
import telecom.sudparis.eu.paas.core.server.xml.environment.ConfigurationType;
import telecom.sudparis.eu.paas.core.server.xml.environment.EntryType;
import telecom.sudparis.eu.paas.core.server.xml.environment.EnvironmentType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationManifest;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasApplicationType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasEnvironmentNodeType;
import telecom.sudparis.eu.paas.core.server.xml.manifest.PaasEnvironmentType;


/**
 * 
 * @author Mehdi Ahmed-Nacer (Telecom SudParis)
 * 
 */
public class CloudFoundaryManageEnvironment {


	/**
	 * ressource bundle to get the connexion credentials
	 */
	private static ResourceBundle rb = ResourceBundle
			.getBundle("telecom.sudparis.eu.paas.core.server.ressources.credentials_cf");

	/**
	 * The public CF-PaaS API URL
	 */
	private static String apiUrl = rb.getString("api.public.url");

	/**
	 * An element to display The response
	 */
	private OperationResponse or = new OperationResponse();

	/**
	 * An element to display Errors
	 */
	private Error error = new Error();

	/**
	 * The Frameworks supported in the current implementation
	 */
	public static final String JAVA_WEB = "java_web";
	public static final String SPRING_WEB = "spring_web/1.0";
	public static final String NODE = "node";

	/**
	 * The Runtimes supported in the current implementation
	 */
	public static final String JAVA6 = "java";
	public static final String JAVA7 = "java7";


	public CloudFoundaryManageEnvironment(){
	}

	public EnvironmentType createEnvironment(String environmentTemplateDescriptor) {
		try {
			if (environmentTemplateDescriptor != null) {
				InputStream is = new ByteArrayInputStream(
						environmentTemplateDescriptor.getBytes());

				JAXBContext jaxbContext;
				PaasApplicationManifest manifest = new PaasApplicationManifest();
				try {
					jaxbContext = JAXBContext
							.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");
					Unmarshaller jaxbUnmarshaller = jaxbContext
							.createUnmarshaller();

					JAXBElement<PaasApplicationManifest> root = jaxbUnmarshaller
							.unmarshal(new StreamSource(is),
									PaasApplicationManifest.class);
					manifest = root.getValue();

				} catch (JAXBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				PaasEnvironmentType envType=manifest.getPaasApplication().
						getPaasEnvironment().get(0);
				// retrieve envName
				String envName = envType.getName();

				// retrieve memory
				long envMemory = Long.parseLong(envType.getPaasEnvironmentTemplate().getMemory());

				// retrieve envDescription
				String envDescription = envType.getPaasEnvironmentTemplate().getDescription();

				// We suppose that we associate the runtime java and the
				// framework "java_web" to the container tomcat
				// TODO consider the case of several containers
				List<String> containerNames = new ArrayList<String>();
				List<String> containerVersions = new ArrayList<String>();
				// containerNames.add("tomcat");// we consider only one
				// container
				List<String> serviceNames = new ArrayList<String>();
				List<PaasEnvironmentNodeType> lstNodes =envType.getPaasEnvironmentTemplate()
						.getPaasEnvironmentNode();

				for (PaasEnvironmentNodeType node : lstNodes) {
					// If no container was specified in the Environment
					// Descriptor, Tomcat will be choosen
					if (node.getContentType().equals("container")) {
						containerNames.add(node.getName());
						containerVersions.add(node.getVersion());
					} else if (node.getContentType().equals("database"))
						serviceNames.add(node.getName());
				}

				// TODO this value may be useful when handling several
				// containers
				int containerIndex = 0;
				Staging s = new Staging(null, null);

				for (String cName : containerNames) {
					if (cName.equals("tomcat")) {
						if (containerVersions.get(containerIndex).startsWith(
								"6")) {
							s = new Staging(null, null);


						} else if (containerVersions.get(containerIndex)
								.startsWith("7")) {
							s = new Staging(null, null);

						}

					} else if (cName.equals("spring")) {

						s = new Staging(null, null);


					} else if (cName.equals("node.js")) {

						s = new Staging(null, null);

					}
					containerIndex++;
				}

				// retrieve the environment configuration (previously called
				// staging)
				// the different configuration elements will be saved in a
				// ConfigurationType element
				ConfigurationType conf = new ConfigurationType();

				// get the command
				EntryType command = new EntryType();
				command.setKey("command");
				command.setValue(s.getCommand());
				conf.getEntry().add(command);

				// get the service Names
				for (String service : serviceNames) {
					EntryType e = new EntryType();
					e.setKey("service");
					e.setValue(service);
					conf.getEntry().add(e);
				}

				// generate envID
				Long id = getNextId();

				EnvironmentType env = new EnvironmentType();
				env.setEnvId((int) (long) id);
				// env.setEnvId(Long.toString(id));
				env.setEnvName(envName);
				env.setEnvDesc(envDescription);
				env.setConfiguration(conf);
				// env.setStaging(staging);
				// env.setServiceNames(serviceNames);
				env.setEnvMemory((int) (long) envMemory);

				// add the environment STATE and hplinks links
				LinksListType linksList = new LinksListType();
				linksList = EnvironmentLinkGenerator.addGetEnvLink(linksList,
						Long.toString(id), apiUrl);
				linksList = EnvironmentLinkGenerator.addGetDeployedAppLink(
						linksList, Long.toString(id), apiUrl);
				linksList = EnvironmentLinkGenerator.addGetEnvLink(linksList,
						Long.toString(id), apiUrl);
				linksList = EnvironmentLinkGenerator.addGetEnvsLink(linksList,
						apiUrl);
				linksList = EnvironmentLinkGenerator.addGetInfoLink(linksList,
						apiUrl);
				linksList = EnvironmentLinkGenerator.addnewEnvLink(linksList,
						apiUrl);

				linksList = EnvironmentLinkGenerator.addDestroyEnvLink(
						linksList, Long.toString(id), apiUrl);
				linksList = EnvironmentLinkGenerator.addUpdateEnvLink(
						linksList, Long.toString(id), apiUrl);

				env.setLinksList(linksList);

				EnvironmentPool.INSTANCE.add(env);

				return env;

			} else {
				System.out
				.println("Failed to retrieve the cloud Environment Descriptor");
				error.setValue("Failed to retrieve the cloud Environment Descriptor.");
				return null;
			}
		} catch (Exception e) {
			System.out.println("Failed to create the environment: "
					+ e.getMessage());
			e.printStackTrace();
			error.setValue("Failed to create the environment: "
					+ e.getMessage());
			return null;
		}

	}
	
	// private methods
	private synchronized Long getNextId() {
		return new Long(EnvironmentPool.INSTANCE.getNextID());
	}

}
